#include "io.h"
#include <stdio.h>
#include <stdbool.h>
#include "system.h"
#include "alt_types.h"
#include "sys/alt_irq.h"
#include "altera_avalon_timer_regs.h"
#include "altera_avalon_timer.h"

// create standard embedded type definitions
typedef   signed char   sint8;              // signed 8 bit values
typedef unsigned char   uint8;              // unsigned 8 bit values
typedef   signed short  sint16;             // signed 16 bit values
typedef unsigned short  uint16;             // unsigned 16 bit values
typedef   signed long   sint32;             // signed 32 bit values
typedef unsigned long   uint32;             // unsigned 32 bit values
typedef         float   real32;             // 32 bit real values

// Numbers
#define DISP_ZERO 	0b1000000
#define DISP_ONE	0b1111001
#define DISP_TWO	0b0100100
#define DISP_THREE	0b0110000
#define DISP_FOUR	0b0011001
#define DISP_FIVE	0b0010010
#define DISP_SIX	0b0000010
#define DISP_SEVEN	0b1111000
#define DISP_EIGHT	0b0000000
#define DISP_NINE	0b0010000
#define COUNT_MAX	9
#define COUNT_MIN	0
#define INTERRUPT_OFFSET    8
#define EDGE_OFFSET	    12
#define BUTTON_ONE	0b0010
#define SW_ZERO		0b00000001

//set up pointers to peripherals
uint32* TimerPtr    = (uint32*)TIMER_0_BASE;
unsigned char* LedPtr      = (unsigned char*)LEDS_BASE;
uint8* HexPtr	= (uint8*)HEX0_BASE;
uint8* SwitchesPtr = (uint8*)SWITCHES_BASE;
uint8* ButtonsPtr = (uint8*)BUTTONS_BASE;
static uint8 displayIndex = 0;
int displayValues[] = {DISP_ZERO, DISP_ONE, DISP_TWO, DISP_THREE, DISP_FOUR, DISP_FIVE, DISP_SIX, DISP_SEVEN, DISP_EIGHT, DISP_NINE};

void isr(void *context)
{
    //Reset edge capture
    *(ButtonsPtr + EDGE_OFFSET) = 0;
    //Check for press or release
    //stop at 9/0
    //if switch0 is high, increment
    if(((*SwitchesPtr & SW_ZERO) == 1) && (displayIndex < COUNT_MAX)){
        displayIndex++;
    //if switch0 is low, decrement
    } else if (((*SwitchesPtr & SW_ZERO) == 0) && (displayIndex > COUNT_MIN)){
        displayIndex--;
    }
    *HexPtr = displayValues[displayIndex];
    return;
}
void timer_isr(void *context)
{
    unsigned char current_val;
    *TimerPtr = 0;
    current_val = *LedPtr; /* read the leds */
    *LedPtr = current_val + 0x1;  /* change the display */
    return;
}

int main(void)
{
    //Set Button ISR to listen to key1
    *(ButtonsPtr + INTERRUPT_OFFSET) = BUTTON_ONE;
    //Register Button ISR
    alt_ic_isr_register(BUTTONS_IRQ_INTERRUPT_CONTROLLER_ID,BUTTONS_IRQ,isr,0,0);
    alt_ic_isr_register(TIMER_0_IRQ_INTERRUPT_CONTROLLER_ID,TIMER_0_IRQ,timer_isr,0,0);
    //Default Value
    *HexPtr = DISP_ZERO;
    *LedPtr = 0; 
     
    while(true){}
    return 0;
}

